/******************************************************/
/* AVR101 "High endurance EEPROM storage"             */
/*                                                    */
/* Filename: High_Endurance_EEPROM.c                  */
/* Date: 2002.08.15                                   */
/* Author: jllassen                                   */
/*                                                    */
/* Modified: 2003.10.06 (raapeland)                   */
/*                                                    */
/* Compiler: IAR EWAVR 2.26C                          */
/******************************************************/

#include <iom16.h>

/******************************************************/
/* Define the number of levels in the buffer, */
/* - eight levels will guarantee 800k writing onf the parameter */
/******************************************************/
#define EE_PARAM_BUFFER_SIZE  8
#define EE_STATUS_BUFFER_SIZE  EE_PARAM_BUFFER_SIZE

/******************************************************/
/* List of parameters */
/******************************************************/
#define EE_PARAM_1  0
#define EE_PARAM_2 (EE_PARAM_1 + EE_PARAM_BUFFER_SIZE + EE_STATUS_BUFFER_SIZE)
//#define EE_PARAM_3 (EE_PARAM_2 + EE_PARAM_BUFFER_SIZE + EE_STATUS_BUFFER_SIZE)  ... and so forth

/******************************************************/
/* prototyping of functions used*/
/******************************************************/
void findCurrentEepromAddr( unsigned int *EeBufPtr );
char EeReadBuffer( unsigned int address );
void EeWriteBuffer( unsigned char parameter, unsigned int *address, unsigned char data );

/******************************************************/
/* Global variables:                                  */
/* Initialize the parameter buffer pointers to be able to resume at the right location.*/
/******************************************************/
unsigned int EeBufPtrParam1 = EE_PARAM_1; 
unsigned int EeBufPtrParam2 = EE_PARAM_2; 

void findCurrentEepromAddr( unsigned int *EeBufPtr )
{
  unsigned char temp;
  unsigned int EeBufEnd;

  *EeBufPtr += EE_PARAM_BUFFER_SIZE;              // Point the the status buffer
  EeBufEnd = *EeBufPtr + EE_STATUS_BUFFER_SIZE;   // The first address outside the buffer

  /* Identify the last writen element of the status buffer */
  do{
    if( *EeBufPtr == EeBufEnd )
      break;
    temp = EeReadBuffer( (*EeBufPtr)++ );
  }while ( EeReadBuffer( *EeBufPtr ) == temp +1 );

  *EeBufPtr -=  (EE_PARAM_BUFFER_SIZE + 1);       // Point to the last used element of the parameter buffer
}

char EeReadBuffer( unsigned int address )
{
  while(EECR & (1<<EEWE));
  EEAR = address;
  EECR |= (1<<EERE);
  return EEDR;
}  

void EeWriteBuffer( unsigned char parameter, unsigned int *address, unsigned char data )
{
  unsigned char EeOldStatusValue;

  /* Store the old status value and move pointer to the next element in the buffer */
  EeOldStatusValue = EeReadBuffer( *address + EE_PARAM_BUFFER_SIZE);
  (*address)++;
  if( *address == parameter + EE_PARAM_BUFFER_SIZE )
  {
    *address = parameter;
  }
  
  // If self-programming is used in the application, insert code here to wait for any
  // self-programming operations to finish before writing to the EEPROM.
  // Also make sure that interrupts are disabled during EEPROM write if interrupts
  // are used in the application, otherwise the 4-cycle limit could be exceeded.

  /* Update the parameter in the EEPROM buffer */
  while(EECR & (1<<EEWE));
  EEAR = *address;
  EEDR = data;
  EECR |= (1<<EEMWE);
  EECR |= (1<<EEWE);

  /* Update the status buffer */
  while(EECR & (1<<EEWE));
  EEAR = *address + EE_PARAM_BUFFER_SIZE;
  EEDR = EeOldStatusValue + 1;
  EECR |= (1<<EEMWE);
  EECR |= (1<<EEWE);
}

/* Appliction main routine - to show how the EEPROM buffers are used */
void main ( void )
{
  unsigned char counter;

  findCurrentEepromAddr( &EeBufPtrParam1 );
  findCurrentEepromAddr( &EeBufPtrParam2 );
  
  for( counter = 0 ; counter < 50; counter++)
  {
    EeWriteBuffer( EE_PARAM_1, &EeBufPtrParam1, counter );
    EeWriteBuffer( EE_PARAM_2, &EeBufPtrParam2, counter*2 );
  }
}
